-- kEnergy
-- Simple energy ticker addon inspired by GonffBar

local name = 'kEnergy'
local ticktime = 2
local fontsize = 12

local function keys(t, k)
    local nk = next(t, k)
    if nk == nil then
        return
    end
    return nk, keys(t, nk)
end
local function empty() return '' end


local candy = AceLibrary('CandyBar-2.0')
local paint = AceLibrary('PaintChips-2.0')
local surface = AceLibrary('Surface-1.0')
local L = AceLibrary('AceLocale-2.2'):new(name)
local addon = AceLibrary('AceAddon-2.0'):new('AceDB-2.0', 'AceEvent-2.0', 'AceConsole-2.0', 'CandyBar-2.0', 'Metrognome-2.0', 'AceDebug-2.0')
local BS = AceLibrary("Babble-Spell-2.2")
local Aura = AceLibrary("SpecialEvents-Aura-2.0")

function addon:OnInitialize()
    surface:Register("BantoBar", "Interface\\Addons\\"..name.."\\Textures\\banto")
    surface:Register("Smooth", "Interface\\Addons\\"..name.."\\Textures\\smooth")
    surface:Register("Perl", "Interface\\Addons\\"..name.."\\Textures\\perl")
    surface:Register("Glaze", "Interface\\Addons\\"..name.."\\Textures\\glaze")
    surface:Register("Charcoal", "Interface\\Addons\\"..name.."\\Textures\\Charcoal")
    surface:Register("Otravi", "Interface\\Addons\\"..name.."\\Textures\\otravi")
    surface:Register("Striped", "Interface\\Addons\\"..name.."\\Textures\\striped")
    surface:Register("NotSteel", "Interface\\Addons\\"..name.."\\Textures\\NotSteel")

    self.options = {
        type = 'group',
        args = {
            width = {
                name  = L['Width'],
                desc  = L['Set the width of the bar.'],
                type  = 'range',
                min   = 25,
                max   = 400,
                get   = function() return self.db.profile.width end,
                set   = function(w)
                    self.db.profile.width = w
                    self.frame:SetWidth(w)
                    self:SetCandyBarWidth(name, w)

                    if not self.locked then
                        candy:Update(name)
                    end
                end,
            },
            height = {
                type  = 'range',
                name  = L['Height'],
                desc  = L['Set the height of the bar.'],
                min   = 5,
                max   = 100,
                get   = function() return self.db.profile.height end,
                set   = function(h)
                    self.db.profile.height = h
                    self.frame:SetHeight(h)
                    self:SetCandyBarHeight(name, h)

                    if not self.locked then
                        candy:Update(name)
                    end
                end,
            },
            shown = {
                type = 'text',
                name = L['Shown'],
                desc = L['Select when the bar will be shown.'],
                get = function() return self.db.profile.tickstate end,
                set = function(v) self.db.profile.tickstate = v end,
                validate = {'always', 'stealth', 'energyticking'},
            },
            texture = {
                type = 'text',
                name = L['Texture'],
                desc = L['Set the texture of the bar.'],
                get  = function() return self.db.profile.texture end,
                set  = function(v)
                    self.db.profile.texture = v
                    self:SetCandyBarTexture(name, surface:Fetch(v))
                end,
                validate = surface:List(),
            },
            color = {
                type = 'text',
                name = L['Color'],
                desc = L['Set the color of the bar.'],
                get  = function() return self.db.profile.color end,
                set  = function(v)
                    self.db.profile.color = v
                    self:SetCandyBarColor(name, v)
                end,
                validate = {keys(paint.vars.hex)},
            },
            lock = {
                type = 'toggle',
                name = L['Lock'],
                desc = L['Lock the bar.'],
                get  = function() return self.locked end,
                set  = function(state) self:Locked(state) end,
            },
            tickoffset = {
                type = 'text',
                name = L['Modify tick time offset'],
                desc = L['Manually modify the offset for the simulated tick time (not persistant).'],
                usage = L['Input a number'],
                get  = function()
                    return self.tickoffset
                end,
                set  = function(v)
                    self.tickoffset = tonumber(v)
                    self:setRate()
                end,
                validate = function(v) return tonumber(v) end,
            },
            resetpos = {
                type = 'execute',
                name = L['Reset position'],
                desc = L['Reset bar position.'],
                func = function()
                    self.frame:ClearAllPoints()
                    self.frame:SetPoint('CENTER', UIParent, 'CENTER', 0, 0)
                end,
            },
        }
    }
    self.locked = true

    self.playerclass = select(2, UnitClass('player'))
    self:RegisterDB('kEnergySaved')
    self:RegisterDefaults('profile', {
        width     = 128,
        height    = 18,
        texture   = 'BantoBar',
        tickstate = 'energyticking',
        color     = self.playerclass:lower(),
    })
    if not self.db.char.initialized then
        self.db.char.initialized = true
        self:SetProfile('class')
    end
    self:RegisterChatCommand({'/kenergy', '/ke'}, self.options)
end

function addon:Locked(state)
    self.locked = state

    if self.locked then
        self:StopCandyBar(name)
        self:SetCandyBarText(name, '')
    else
        self:StartCandyBar(name)
        self:SetCandyBarTimeLeft(name, ticktime)
        self:SetCandyBarText(name, name)
        self:PauseCandyBar(name)
        candy:Update(name)
    end

    self.frame:EnableMouse(not state)
end

function addon:OnEnable()
    if self.playerclass == 'DRUID' then
        self:RegisterEvent('SpecialEvents_UnitBuffGained', function(unit, buff)
            if unit == 'player' and buff == BS['Cat Form'] then
                self:StartMetro(name)
            end
        end)
        self:RegisterEvent('SpecialEvents_UnitBuffLost', function(unit, buff)
            if unit == 'player' and buff == BS['Cat Form'] then
                prevtime = nil
                self.energyticking = nil
                self:StopMetro(name)
                self:StopCandyBar(name)
            end
        end)

        self.ticktime = 2.0
    elseif self.playerclass == 'ROGUE' then
        self:RegisterEvent('SpecialEvents_UnitBuffGained', function(unit, buff)
            if unit == 'player' and buff == BS['Adrenaline Rush'] then
                self.ticktime = 1.0
                self:setRate()
            end
        end)
        self:RegisterEvent('SpecialEvents_UnitBuffLost', function(unit, buff)
            if unit == 'player' and buff == BS['Adrenaline Rush'] then
                self.ticktime = 2.0
                self:setRate()
            end
        end)

        if Aura:UnitHasBuff(BS['Adrenaline Rush']) then
            self.ticktime = 1.0
        else
            self.ticktime = 2.0
        end
    else
        DisableAddOn(name)
        return
    end

    self:RegisterEvent('Surface_Registered', function()
        self.options.args.texture.validate = surface:List()
    end)

    self.maxen  = UnitManaMax('player')
    self.energy = UnitMana('player')

    self:RegisterCandyBar(name, self.ticktime, '', nil, self.db.profile.color)
    self:SetCandyBarHeight(name, self.db.profile.height)
    self:SetCandyBarWidth(name, self.db.profile.width)
    self:SetCandyBarTexture(name, surface:Fetch(self.db.profile.texture))
    self:SetCandyBarTimeFormat(name, empty)
    self:SetCandyBarFontSize(name, fontsize)

    self:CreateEventsFrame()
    self:CreateFrame()
    self:RestorePosition()

    self:RegisterMetro(name, self.tick, self.ticktime, self)
end


_G[name] = addon
