local name = 'kEnergy'
local Aura = AceLibrary("SpecialEvents-Aura-2.0")
local BS = AceLibrary("Babble-Spell-2.2")
local G = AceLibrary("Gratuity-2.0")

-- Scan spellbook to determine the energy costs of our abilities
function kEnergy:ScanSpellbook()
    local i = 1
    local sName, sRank, sCost

    repeat
        local sIndex, sName, sRank = i, GetSpellName(i, BOOKTYPE_SPELL)
        if sName then
            G:SetSpell(i, BOOKTYPE_SPELL)
            local sCost = G:FindDeformat(ENERGY_COST, 2, 2, false, true)
            if sCost then
                self.spellcosts[sName] = sCost
            end
        end
        i = i + 1
    until not sName
end

function kEnergy:CreateEventsFrame()
    local prevtime
    local count, sum, sumsq = 0, 0, 0
    self.pending = 0
    self.tickoffset = 0

    -- self:SetDebugging(true)
    -- self:SetDebugLevel(3)

    self.spellcosts = {}
    self:ScanSpellbook()
    self:RegisterEvent("LEARNED_SPELL_IN_TAB", "ScanSpellbook")

    self:RegisterEvent('UNIT_MAXENERGY', function(unit)
        if unit == 'player' then
            self.maxen = UnitManaMax(unit)
        end
    end)

    -- Track energy usage
    self:RegisterEvent('COMBAT_TEXT_UPDATE', function(msgtype, amount, ...)
        if msgtype == 'ENERGY' then
            self.pending = self.pending + amount
            self:LevelDebug(2, "+%s", amount)
        end
    end)
    self:RegisterEvent('UNIT_SPELLCAST_SUCCEEDED', function(unit, spell, rank)
        local cost = self.spellcosts[spell]
        if cost then
            self.pending = self.pending - cost
            self:LevelDebug(2, "-%s", cost)
        end
    end)

    -- Anticipate the approaching energy max due to leveling
    self:RegisterEvent('PLAYER_LEVEL_UP', function()
        self:LevelDebug(2, 'Player leveled, anticipate the final UNIT_ENERGY')
        self.pending = self.maxen - self.energy
    end)

    self:RegisterEvent('PLAYER_DEAD', function()
        self:LevelDebug(2, 'Player died, resetting tick cycle, anticipate the final UNIT_ENERGY')
        self.pending = -self.energy
        prevtime = nil
        self.energyticking = nil
    end)

    -- Use a seperate frame for the energy tick, to ensure that no possible
    -- overhead in the event handling introduces a delay in the tick time
    -- measurement
    self.events = CreateFrame("Frame", nil, UIParent)
    self.events:SetScript("OnEvent", function(self, event, ...)
        kEnergy.events[event](kEnergy, ...)
    end)
    self.events:RegisterEvent('UNIT_ENERGY')

    -- I hate you, Blizzard
    local potencyamount
    local version = GetBuildInfo()
    local maj, min, rev = strsplit(".", version)
    if tonumber(maj) == 2 and tonumber(rev) < 3 then
        local numPoints = select(5, GetTalentInfo(2, 23))
        if numPoints then
            potencyamount = numPoints * 3
        end
    end

    function self.events:UNIT_ENERGY(unit)
        local time = GetTime()
        local olden = self.energy

        self.energy = UnitMana(unit)
        local diff = self.energy - (olden + self.pending)
        self.pending = 0

        if diff > -1 and diff < 1 then
            -- Energy alterations were fully anticipated, not a tick
            return
        end

        if self.energy == self.maxen then
            self.energyticking = nil

            if olden + self.pending >= self.maxen then
                -- Pending energy alterations caused us to hit max energy--not a tick
                return
            end
        elseif potencyamount and diff < (potencyamount+1) and diff > (potencyamount-1) then
            self:LevelDebug(2, "Combat potency gain, not a tick")
            return
        elseif diff > 21 or diff < 19 then
            self:LevelDebug(1, "Warning! Unexpected energy difference of %s, assuming not a tick.", diff)
            return
        else
            self.energyticking = true
        end

        -- Attempt to keep a running standard deviation, so we can throw away
        -- outliers before they screw up our numbers.
        if prevtime then
            local timediff = time - prevtime - self.ticktime
            local sd   = math.sqrt((sumsq - (sum*sum/count))/(count - 1))
            if sd ~= 0 and
                (timediff > (self.tickoffset + 3*sd) or
                 timediff < (self.tickoffset - 3*sd)) then
                -- outlier, throwing out
                self:LevelDebug(1, "Outlier detected in tick times, throwing away timediff of %s.", timediff)
            else
                sum   = sum + timediff
                sumsq = sum + timediff^2
                count = count + 1
                self.tickoffset  = sum / count
            end
        end

        if self.energy == self.maxen then
            prevtime = nil
        else
            prevtime = time
        end

        self:tick(0, true)
    end

    function kEnergy:setRate()
        self:ChangeMetroRate(name, self.ticktime + self.tickoffset)
        local running = select(3, self:MetroStatus(name))
        if not running then
            self:StartMetro(name)
        end

        self:SetCandyBarTime(name, self.ticktime + self.tickoffset)
    end
end

-- NOTE: Metro ticker is always running, even when the ticks are forced via
-- UNIT_ENERGY.  This is necessary for the case where we hit max mana via a
-- means other than a tick, so that final UNIT_ENERGY is an unsuitable point
-- to start the metrognome ticker.
function kEnergy:tick(elapsed, force)
    self:setRate()
    if self.locked then
        if force or
           not self.energyticking and (
               (self.db.profile.tickstate == 'always') or
               (self.db.profile.tickstate == 'stealth' and ((self.playerclass == 'ROGUE' and Aura:UnitHasBuff('player', BS['Stealth'])) or (self.playerclass == 'DRUID' and Aura:UnitHasBuff('player', BS['Prowl']))))
           ) then
            self:StartCandyBar(name)
        end
    end
end
